import { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { User, Accounts, Cards, Transfers } from '../types';
import axios from 'axios';

import TopBar from '../components/TopBar';

export default function Home () {
    const navigate = useNavigate();
    const [user, setUser] = useState<User>();
    const [transfers, setTransfers] = useState<Transfers>([]);
    const [accounts, setAccounts] = useState<Accounts>([]);
    const [cards, setCards] = useState<Cards>([]);
    const [accountId, setAccountId] = useState<number>(0);
    const [accountToId, setAccountToId] = useState<number>(0);
    const [amount, setAmount] = useState<number>(0);
    const [message, setMessage] = useState<string>("");
    const token = localStorage.getItem("token");

    function sendMoney (e: React.FormEvent) {
        e.preventDefault();
        
        axios
            .post("/api/@me/send-money", {
                token: token,
                account_from_id: accountId,
                account_to_id: accountToId,
                amount: amount,
                name: message
            })
            .then(() => {
                axios
                    .post("/api/@me/accounts", { token: token })
                    .then((response) => {
                        setAccounts(response.data);
                        if (response.data.length > 0) {
                            setAccountId(response.data[0].id);
                        }
                    })
                    .catch(() => {
                        console.error("Failed to fetch accounts");
                    });
                })
                .catch((error) => {
                    console.error("Failed to send money", error.response.data);
                });
    }

    useEffect(() => {
        const storedToken = localStorage.getItem("token");
        if (storedToken) {
            axios
                .post("/api/@me", { token: storedToken })
                .then((response) => {
                    setUser(response.data);
                })
                .catch(() => {
                    navigate("/login");
                });

            axios
                .post("/api/@me/transfers", { token: storedToken })
                .then((response) => {
                    setTransfers(response.data);
                })
                .catch(() => {
                    console.error("Failed to fetch transfers");
                });

            axios
                .post("/api/@me/accounts", { token: storedToken })
                .then((response) => {
                    setAccounts(response.data);
                    if (response.data.length > 0) {
                        setAccountId(response.data[0].id);
                    }
                })
                .catch(() => {
                    console.error("Failed to fetch accounts");
                });

            axios
                .post("/api/@me/cards", { token: storedToken })
                .then((response) => {
                    setCards(response.data);
                })
                .catch(() => {
                    console.error("Failed to fetch cards");
                });
        } else {
            navigate("/login");
        }
    }, []);

    if (!user) {
        return (
            <div className='bank-page'>
                <TopBar user={user} />
                <div className='bank-section'>
                    <h1>Loading...</h1>
                </div>
            </div>
        );
    }

    return (
        <div className='bank-page'>
            <TopBar user={user} />
            <div className='bank-section'>
                <h1>Home</h1>
                <p>Welcome {user.name} {user.lastname}</p>
            </div>
            <div className='bank-section'>
                <h2>Your Transfers</h2>
                {transfers.length > 0 ? (
                    <ul>
                        {transfers.map((transfer) => (
                            <li key={transfer.id}>
                                Transfer ID: {transfer.id}, Amount: {transfer.value}
                            </li>
                        ))}
                    </ul>
                ) : (
                    <p>No transfers found.</p>
                )}
            </div>
            {accounts.length > 0 && (
                <div className='bank-section'>
                    <h2>Send money</h2>
                    <form onSubmit={sendMoney}>
                        <select name="" id="" value={String(accountId)} onChange={(e) => setAccountId(Number(e.target.value))}>
                            {accounts.map((account: any) => (
                                <option key={account.id} value={account.id}>
                                    {account.name}
                                </option>
                            ))}
                        </select>
                        <input type="number" placeholder="Amount" onChange={(e) => setAmount(Number(e.target.value))} value={amount}/>
                        <input type="text" placeholder="Recipient Account ID" onChange={(e) => setAccountToId(Number(e.target.value))} value={accountToId}/>
                        <input type="text" placeholder="Message" onChange={(e) => setMessage(e.target.value)} value={message}/>
                        <button type="submit">Send</button>
                    </form>
                </div>
            )}
            <div className='bank-section'>
                <h2>Your Accounts</h2>
                {accounts.length > 0 ? (
                    <ul>
                        {accounts.map((account) => (
                            <li key={account.id}>
                                Account ID: {account.id}, Balance: {account.balance}, Interest: {account.interest}
                            </li>
                        ))}
                    </ul>
                ) : (
                    <p>No accounts found.</p>
                )}
            </div>
            <div className='bank-section'>
                <h2>Your Cards</h2>
                {cards.length > 0 ? (
                    <ul>
                        {cards.map((card) => (
                            <li key={card.id}>
                                Card ID: {card.id}, Account ID: {card.account_id}, Number: {card.number}, Expiration: {card.expiration}
                            </li>
                        ))}
                    </ul>
                ) : (
                    <p>No cards found.</p>
                )}
            </div>
        </div>
    )
}